def Behavior_UI():
    from PymoNNto.Exploration.Behavior_UI import Behavior_UI
    Behavior_UI().show()

def Execute_UI():
    from PymoNNto.Exploration.Execute_UI import Execute_UI
    Execute_UI().show()

def Overview_UI():
    from PymoNNto.Exploration.Overview_UI import Overview_UI
    Overview_UI().show()

def Evolution_UI():
    from PymoNNto.Exploration.Evolution.Evolution_UI import Evolution_UI
    Evolution_UI().show()
